@@ -82,6 +82,7 @@ camel_imap4_engine_init (CamelIMAP4Engine *engine, CamelIMAP4EngineClass *klass)
 	engine->level = CAMEL_IMAP4_LEVEL_UNKNOWN;
 	
 	engine->session = NULL;
+	engine->service = NULL;
 	engine->url = NULL;
 	
 	engine->istream = NULL;
@@ -134,9 +135,6 @@ camel_imap4_engine_finalize (CamelObject *object)
 	CamelIMAP4Engine *engine = (CamelIMAP4Engine *) object;
 	EDListNode *node;
 	
-	if (engine->session)
-		camel_object_unref (engine->session);
-	
 	if (engine->istream)
 		camel_object_unref (engine->istream);
 	
@@ -164,22 +162,21 @@ camel_imap4_engine_finalize (CamelObject *object)
 
 /**
  * camel_imap4_engine_new:
- * @session: session
- * @url: service url
+ * @service: service
  *
  * Returns a new imap4 engine
  **/
 CamelIMAP4Engine *
-camel_imap4_engine_new (CamelSession *session, CamelURL *url)
+camel_imap4_engine_new (CamelService *service)
 {
 	CamelIMAP4Engine *engine;
 	
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
+	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
 	
 	engine = (CamelIMAP4Engine *) camel_object_new (CAMEL_TYPE_IMAP4_ENGINE);
-	camel_object_ref (session);
-	engine->session = session;
-	engine->url = url;
+	engine->session = service->session;
+	engine->url = service->url;
+	engine->service = service;
 	
 	return engine;
 }
@@ -264,7 +261,7 @@ camel_imap4_engine_capability (CamelIMAP4Engine *engine, CamelException *ex)
 	CamelIMAP4Command *ic;
 	int id, retval = 0;
 	
-	ic = camel_imap4_engine_queue (engine, NULL, "CAPABILITY\r\n");
+	ic = camel_imap4_engine_prequeue (engine, NULL, "CAPABILITY\r\n");
 	
 	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
@@ -298,9 +295,9 @@ camel_imap4_engine_namespace (CamelIMAP4Engine *engine, CamelException *ex)
 	int id, i;
 	
 	if (engine->capa & CAMEL_IMAP4_CAPABILITY_NAMESPACE) {
-		ic = camel_imap4_engine_queue (engine, NULL, "NAMESPACE\r\n");
+		ic = camel_imap4_engine_prequeue (engine, NULL, "NAMESPACE\r\n");
 	} else {
-		ic = camel_imap4_engine_queue (engine, NULL, "LIST \"\" \"\"\r\n");
+		ic = camel_imap4_engine_prequeue (engine, NULL, "LIST \"\" \"\"\r\n");
 		camel_imap4_command_register_untagged (ic, "LIST", camel_imap4_untagged_list);
 		ic->user_data = array = g_ptr_array_new ();
 	}
@@ -438,8 +435,8 @@ static struct {
 	const char *name;
 	guint32 flag;
 } imap4_capabilities[] = {
-	{ "IMAP44",         CAMEL_IMAP4_CAPABILITY_IMAP44         },
-	{ "IMAP44REV1",     CAMEL_IMAP4_CAPABILITY_IMAP44REV1     },
+	{ "IMAP4",         CAMEL_IMAP4_CAPABILITY_IMAP4         },
+	{ "IMAP4REV1",     CAMEL_IMAP4_CAPABILITY_IMAP4REV1     },
 	{ "STATUS",        CAMEL_IMAP4_CAPABILITY_STATUS        },
 	{ "NAMESPACE",     CAMEL_IMAP4_CAPABILITY_NAMESPACE     },
 	{ "UIDPLUS",       CAMEL_IMAP4_CAPABILITY_UIDPLUS       },
@@ -496,11 +493,11 @@ engine_parse_capability (CamelIMAP4Engine *engine, int sentinel, CamelException
 	camel_imap4_stream_unget_token (engine->istream, &token);
 	
 	/* figure out which version of IMAP4 we are dealing with */
-	if (engine->capa & CAMEL_IMAP4_CAPABILITY_IMAP44REV1) {
-		engine->level = CAMEL_IMAP4_LEVEL_IMAP44REV1;
+	if (engine->capa & CAMEL_IMAP4_CAPABILITY_IMAP4REV1) {
+		engine->level = CAMEL_IMAP4_LEVEL_IMAP4REV1;
 		engine->capa |= CAMEL_IMAP4_CAPABILITY_STATUS;
-	} else if (engine->capa & CAMEL_IMAP4_CAPABILITY_IMAP44) {
-		engine->level = CAMEL_IMAP4_LEVEL_IMAP44;
+	} else if (engine->capa & CAMEL_IMAP4_CAPABILITY_IMAP4) {
+		engine->level = CAMEL_IMAP4_LEVEL_IMAP4;
 	} else {
 		engine->level = CAMEL_IMAP4_LEVEL_UNKNOWN;
 	}
@@ -989,15 +986,7 @@ camel_imap4_engine_handle_untagged_1 (CamelIMAP4Engine *engine, camel_imap4_toke
 			
 			engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
 			
-			/* FIXME: emit a "disconnected" signal for our Store?
-			 * The Store could then initiate a reconnect if
-			 * desirable. */
-			
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("IMAP4 server %s unexpectedly disconnected: %s"),
-					      engine->url->host, _("Got BYE response"));
-			
-			return -1;
+			/* we don't return -1 here because there may be more untagged responses after the BYE */
 		} else if (!strcmp ("CAPABILITY", token->v.atom)) {
 			/* capability tokens follow */
 			if (engine_parse_capability (engine, '\n', ex) == -1)
@@ -1184,8 +1173,7 @@ engine_prequeue_folder_select (CamelIMAP4Engine *engine)
 	}
 	
 	/* we need to pre-queue a SELECT */
-	ic = camel_imap4_command_new (engine, ic->folder, "SELECT %F\r\n", ic->folder);
-	camel_imap4_engine_prequeue (engine, ic);
+	ic = camel_imap4_engine_prequeue (engine, (CamelFolder *) ic->folder, "SELECT %F\r\n", ic->folder);
 	ic->user_data = engine;
 	
 	camel_imap4_command_unref (ic);
@@ -1291,7 +1279,7 @@ camel_imap4_engine_iterate (CamelIMAP4Engine *engine)
  * @format: command format
  * @Varargs: arguments
  *
- * Basically the same as #camel_imap4_command_new() except that this
+ * Basically the same as camel_imap4_command_new() except that this
  * function also places the command in the engine queue.
  *
  * Returns the CamelIMAP4Command.
@@ -1302,6 +1290,8 @@ camel_imap4_engine_queue (CamelIMAP4Engine *engine, CamelFolder *folder, const c
 	CamelIMAP4Command *ic;
 	va_list args;
 	
+	g_return_val_if_fail (CAMEL_IS_IMAP4_ENGINE (engine), NULL);
+	
 	va_start (args, format);
 	ic = camel_imap4_command_newv (engine, (CamelIMAP4Folder *) folder, format, args);
 	va_end (args);
@@ -1317,17 +1307,26 @@ camel_imap4_engine_queue (CamelIMAP4Engine *engine, CamelFolder *folder, const c
 /**
  * camel_imap4_engine_prequeue:
  * @engine: IMAP4 engine
- * @ic: IMAP4 command to pre-queue
+ * @folder: IMAP4 folder that the command will affect (or %NULL if it doesn't matter)
+ * @format: command format
+ * @Varargs: arguments
  *
- * Places @ic at the head of the queue of pending IMAP4 commands.
+ * Same as camel_imap4_engine_queue() except this places the new
+ * command at the head of the queue.
+ *
+ * Returns the CamelIMAP4Command.
  **/
-void
-camel_imap4_engine_prequeue (CamelIMAP4Engine *engine, CamelIMAP4Command *ic)
+CamelIMAP4Command *
+camel_imap4_engine_prequeue (CamelIMAP4Engine *engine, CamelFolder *folder, const char *format, ...)
 {
-	g_return_if_fail (CAMEL_IS_IMAP4_ENGINE (engine));
-	g_return_if_fail (ic != NULL);
+	CamelIMAP4Command *ic;
+	va_list args;
 	
-	camel_imap4_command_ref (ic);
+	g_return_val_if_fail (CAMEL_IS_IMAP4_ENGINE (engine), NULL);
+	
+	va_start (args, format);
+	ic = camel_imap4_command_newv (engine, (CamelIMAP4Folder *) folder, format, args);
+	va_end (args);
 	
 	if (e_dlist_empty (&engine->queue)) {
 		e_dlist_addtail (&engine->queue, (EDListNode *) ic);
@@ -1351,6 +1350,10 @@ camel_imap4_engine_prequeue (CamelIMAP4Engine *engine, CamelIMAP4Command *ic)
 			}
 		}
 	}
+	
+	camel_imap4_command_ref (ic);
+	
+	return ic;
 }
 
 
@@ -1377,6 +1380,9 @@ camel_imap4_engine_next_token (CamelIMAP4Engine *engine, camel_imap4_token_t *to
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("IMAP4 server %s unexpectedly disconnected: %s"),
 				      engine->url->host, errno ? g_strerror (errno) : _("Unknown"));
+		
+		engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
+		
 		return -1;
 	}
 	
@@ -1405,6 +1411,8 @@ camel_imap4_engine_eat_line (CamelIMAP4Engine *engine, CamelException *ex)
 						      _("IMAP4 server %s unexpectedly disconnected: %s"),
 						      engine->url->host, errno ? g_strerror (errno) : _("Unknown"));
 				
+				engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
+				
 				return -1;
 			}
 		}
@@ -1438,6 +1446,8 @@ camel_imap4_engine_line (CamelIMAP4Engine *engine, unsigned char **line, size_t
 		if (linebuf != NULL)
 			g_byte_array_free (linebuf, TRUE);
 		
+		engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
+		
 		return -1;
 	}
 	
@@ -1478,6 +1488,8 @@ camel_imap4_engine_literal (CamelIMAP4Engine *engine, unsigned char **literal, s
 		if (literalbuf != NULL)
 			g_byte_array_free (literalbuf, TRUE);
 		
+		engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
+		
 		return -1;
 	}
 	
