@@ -40,6 +40,7 @@
 #include "camel-imap4-stream.h"
 #include "camel-imap4-command.h"
 #include "camel-imap4-utils.h"
+#include "camel-imap4-summary.h"
 
 
 static void camel_imap4_store_class_init (CamelIMAP4StoreClass *klass);
@@ -97,15 +98,16 @@ imap4_hash_folder_name (gconstpointer key)
 		return g_str_hash (key);
 }
 
-static gint
+static int
 imap4_compare_folder_name (gconstpointer a, gconstpointer b)
 {
 	gconstpointer aname = a, bname = b;
-
+	
 	if (g_ascii_strcasecmp (a, "INBOX") == 0)
 		aname = "INBOX";
 	if (g_ascii_strcasecmp (b, "INBOX") == 0)
 		bname = "INBOX";
+	
 	return g_str_equal (aname, bname);
 }
 
@@ -149,13 +151,22 @@ camel_imap4_store_finalize (CamelObject *object)
 	
 	if (store->engine)
 		camel_object_unref (store->engine);
+	
+	g_free (store->storage_path);
 }
 
 
 static void
 imap4_construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, CamelException *ex)
 {
+	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
+	
 	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
+	if (camel_exception_is_set (ex))
+		return;
+	
+	store->storage_path = camel_session_get_storage_path (session, service, ex);
+	store->engine = camel_imap4_engine_new (service);
 }
 
 static char *
@@ -178,18 +189,12 @@ enum {
 #define STARTTLS_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_TLS)
 
 static gboolean
-connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, int try_starttls, CamelException *ex)
+connect_to_server (CamelIMAP4Engine *engine, struct hostent *host, int ssl_mode, int try_starttls, CamelException *ex)
 {
-	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
-	CamelIMAP4Engine *engine;
+	CamelService *service = engine->service;
 	CamelStream *tcp_stream;
 	int port, ret;
 	
-	if (store->engine) {
-		camel_object_unref (store->engine);
-		store->engine = NULL;
-	}
-	
 	port = service->url->port ? service->url->port : 143;
 	
 	if (ssl_mode) {
@@ -215,7 +220,6 @@ connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, in
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
 	
-	fprintf (stderr, "connecting to %s:%d\n", service->url->host, port);
 	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, host, port)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
@@ -231,20 +235,11 @@ connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, in
 		return FALSE;
 	}
 	
-	engine = camel_imap4_engine_new (service->session, service->url);
-	if (camel_imap4_engine_take_stream (engine, tcp_stream, ex) == -1) {
-		camel_object_unref (engine);
-		
+	if (camel_imap4_engine_take_stream (engine, tcp_stream, ex) == -1)
 		return FALSE;
-	}
 	
-	if (camel_imap4_engine_capability (engine, ex) == -1) {
-		camel_object_unref (engine);
-		
+	if (camel_imap4_engine_capability (engine, ex) == -1)
 		return FALSE;
-	}
-	
-	store->engine = engine;
 	
 #ifdef HAVE_SSL
 	if (ssl_mode == USE_SSL_WHEN_POSSIBLE) {
@@ -261,7 +256,7 @@ connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, in
 						      _("Failed to connect to IMAP server %s in secure mode: "
 							"Server does not support STARTTLS"),
 						      service->url->host);
-				goto exception;
+				return FALSE;
 			}
 		}
 	}
@@ -276,7 +271,7 @@ connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, in
 		CamelIMAP4Command *ic;
 		int id;
 		
-		ic = camel_imap4_engine_queue (engine, NULL, "STARTTLS\r\n");
+		ic = camel_imap4_engine_prequeue (engine, NULL, "STARTTLS\r\n");
 		while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 			;
 		
@@ -291,20 +286,13 @@ connect_to_server (CamelService *service, struct hostent *host, int ssl_mode, in
 			
 			camel_imap4_command_unref (ic);
 			
-			goto exception;
+			return FALSE;
 		}
 		
 		camel_imap4_command_unref (ic);
 	}
 	
 	return TRUE;
-	
- exception:
-	
-	camel_object_unref (store->engine);
-	store->engine = NULL;
-	
-	return FALSE;
 #endif /* HAVE_SSL */
 }
 
@@ -320,8 +308,9 @@ static struct {
 };
 
 static gboolean
-connect_to_server_wrapper (CamelService *service, CamelException *ex)
+connect_to_server_wrapper (CamelIMAP4Engine *engine, CamelException *ex)
 {
+	CamelService *service = engine->service;
 	const char *use_ssl;
 	struct hostent *h;
 	int ssl_mode;
@@ -341,19 +330,19 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 	
 	if (ssl_mode == USE_SSL_ALWAYS) {
 		/* First try the ssl port */
-		if (!(ret = connect_to_server (service, h, ssl_mode, FALSE, ex))) {
+		if (!(ret = connect_to_server (engine, h, ssl_mode, FALSE, ex))) {
 			if (camel_exception_get_id (ex) == CAMEL_EXCEPTION_SERVICE_UNAVAILABLE) {
 				/* The ssl port seems to be unavailable, lets try STARTTLS */
 				camel_exception_clear (ex);
-				ret = connect_to_server (service, h, ssl_mode, TRUE, ex);
+				ret = connect_to_server (engine, h, ssl_mode, TRUE, ex);
 			}
 		}
 	} else if (ssl_mode == USE_SSL_WHEN_POSSIBLE) {
 		/* If the server supports STARTTLS, use it */
-		ret = connect_to_server (service, h, ssl_mode, TRUE, ex);
+		ret = connect_to_server (engine, h, ssl_mode, TRUE, ex);
 	} else {
 		/* User doesn't care about SSL */
-		ret = connect_to_server (service, h, USE_SSL_ALWAYS, FALSE, ex);
+		ret = connect_to_server (engine, h, USE_SSL_NEVER, FALSE, ex);
 	}
 	
 	camel_free_host (h);
@@ -400,9 +389,9 @@ sasl_auth (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, const unsigned char
 }
 
 static int
-imap4_try_authenticate (CamelService *service, gboolean reprompt, const char *errmsg, CamelException *ex)
+imap4_try_authenticate (CamelIMAP4Engine *engine, gboolean reprompt, const char *errmsg, CamelException *ex)
 {
-	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
+	CamelService *service = engine->service;
 	CamelSession *session = service->session;
 	CamelSasl *sasl = NULL;
 	CamelIMAP4Command *ic;
@@ -431,18 +420,18 @@ imap4_try_authenticate (CamelService *service, gboolean reprompt, const char *er
 	if (service->url->authmech) {
 		CamelServiceAuthType *mech;
 		
-		mech = g_hash_table_lookup (store->engine->authtypes, service->url->authmech);
+		mech = g_hash_table_lookup (engine->authtypes, service->url->authmech);
 		sasl = camel_sasl_new ("imap4", mech->authproto, service);
 		
-		ic = camel_imap4_engine_queue (store->engine, NULL, "AUTHENTICATE %s\r\n", service->url->authmech);
+		ic = camel_imap4_engine_prequeue (engine, NULL, "AUTHENTICATE %s\r\n", service->url->authmech);
 		ic->plus = sasl_auth;
 		ic->user_data = sasl;
 	} else {
-		ic = camel_imap4_engine_queue (store->engine, NULL, "LOGIN %S %S\r\n",
-					       service->url->user, service->url->passwd);
+		ic = camel_imap4_engine_prequeue (engine, NULL, "LOGIN %S %S\r\n",
+						  service->url->user, service->url->passwd);
 	}
 	
-	while ((id = camel_imap4_engine_iterate (store->engine)) < ic->id && id != -1)
+	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
 	
 	if (sasl != NULL)
@@ -470,38 +459,29 @@ imap4_try_authenticate (CamelService *service, gboolean reprompt, const char *er
 }
 
 static gboolean
-imap4_connect (CamelService *service, CamelException *ex)
+imap4_reconnect (CamelIMAP4Engine *engine, CamelException *ex)
 {
-	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
+	CamelService *service = engine->service;
 	CamelServiceAuthType *mech;
 	gboolean reprompt = FALSE;
 	char *errmsg = NULL;
 	CamelException lex;
 	
-	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
-	if (!connect_to_server_wrapper (service, ex)) {
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+	if (!connect_to_server_wrapper (engine, ex))
 		return FALSE;
-	}
 	
-#define CANT_USE_AUTHMECH (!(mech = g_hash_table_lookup (store->engine->authtypes, service->url->authmech)))
+#define CANT_USE_AUTHMECH (!(mech = g_hash_table_lookup (engine->authtypes, service->url->authmech)))
 	if (service->url->authmech && CANT_USE_AUTHMECH) {
 		/* Oops. We can't AUTH using the requested mechanism */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				      _("Cannot authenticate to IMAP server %s using %s"),
 				      service->url->host, service->url->authmech);
 		
-		camel_object_unref (store->engine);
-		store->engine = NULL;
-		
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
-		
 		return FALSE;
 	}
 	
 	camel_exception_init (&lex);
-	while (imap4_try_authenticate (service, reprompt, errmsg, &lex)) {
+	while (imap4_try_authenticate (engine, reprompt, errmsg, &lex)) {
 		g_free (errmsg);
 		errmsg = g_strdup (lex.desc);
 		camel_exception_clear (&lex);
@@ -511,28 +491,27 @@ imap4_connect (CamelService *service, CamelException *ex)
 	
 	if (camel_exception_is_set (&lex)) {
 		camel_exception_xfer (ex, &lex);
-		camel_object_unref (store->engine);
-		store->engine = NULL;
-		
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
-		
 		return FALSE;
 	}
 	
-	if (camel_imap4_engine_namespace (store->engine, ex) == -1) {
-		camel_object_unref (store->engine);
-		store->engine = NULL;
-		
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
-		
+	if (camel_imap4_engine_namespace (engine, ex) == -1)
 		return FALSE;
-	}
-	
-	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 	
 	return TRUE;
 }
 
+static gboolean
+imap4_connect (CamelService *service, CamelException *ex)
+{
+	gboolean retval;
+	
+	CAMEL_SERVICE_LOCK (service, connect_lock);
+	retval = imap4_reconnect (((CamelIMAP4Store *) service)->engine, ex);
+	CAMEL_SERVICE_UNLOCK (service, connect_lock);
+	
+	return retval;
+}
+
 static gboolean
 imap4_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
@@ -548,8 +527,6 @@ imap4_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 		camel_imap4_command_unref (ic);
 	}
 	
-	camel_object_unref (store->engine);
-	
 	return 0;
 }
 
@@ -564,7 +541,7 @@ imap4_query_auth_types (CamelService *service, CamelException *ex)
 	gboolean connected;
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	connected = connect_to_server_wrapper (service, ex);
+	connected = connect_to_server_wrapper (store->engine, ex);
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 	if (!connected)
 		return NULL;
@@ -693,7 +670,7 @@ imap4_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Cam
 	
 	/* make sure the folder exists - try LISTing it? */
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
-	ic = camel_imap4_engine_queue (engine, NULL, "LIST "" %S\r\n", utf7_name);
+	ic = camel_imap4_engine_queue (engine, NULL, "LIST \"\" %S\r\n", utf7_name);
 	camel_imap4_command_register_untagged (ic, "LIST", camel_imap4_untagged_list);
 	ic->user_data = array = g_ptr_array_new ();
 	g_free (utf7_name);
@@ -771,13 +748,17 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 	CamelFolderInfo *fi = NULL;
 	CamelIMAP4Command *ic;
 	char *utf7_name;
+	CamelURL *url;
 	const char *c;
 	char *name;
+	char sep;
 	int id;
 	
+	sep = imap4_get_path_delim (engine, parent_name);
+	
 	c = folder_name;
 	while (*c != '\0') {
-		if (*c == store->dir_sep || strchr ("#%*", *c)) {
+		if (*c == sep || strchr ("/#%*", *c)) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_PATH,
 					      _("The folder name \"%s\" is invalid because "
 						"it contains the character \"%c\""),
@@ -811,12 +792,19 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 	
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
-		if (!(fi = imap4_get_folder_info (store, name, CAMEL_STORE_FOLDER_INFO_FAST, ex))) {
-			camel_imap4_command_unref (ic);
-			g_free (name);
-			
-			goto done;
-		}
+		url = camel_url_copy (engine->url);
+		camel_url_set_fragment (url, name);
+		
+		c = strrchr (name, '/');
+		
+		fi = g_malloc0 (sizeof (CamelFolderInfo));
+		fi->full_name = name;
+		fi->name = g_strdup (c ? c + 1: name);
+		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+		camel_url_free (url);
+		fi->flags = 0;
+		fi->unread = -1;
+		fi->total = -1;
 		
 		camel_object_trigger_event (store, "folder_created", fi);
 		break;
@@ -825,11 +813,13 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot create folder `%s': Invalid mailbox name"),
 				      name);
+		g_free (name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot create folder `%s': Bad command"),
 				      name);
+		g_free (name);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -837,8 +827,6 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 	
 	camel_imap4_command_unref (ic);
 	
-	g_free (name);
-	
  done:
 	
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
@@ -850,8 +838,12 @@ static void
 imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
-	CamelIMAP4Command *ic;
+	CamelFolder *selected = (CamelFolder *) engine->folder;
+	CamelIMAP4Command *ic, *ic0 = NULL;
+	CamelFolderInfo *fi;
 	char *utf7_name;
+	CamelURL *url;
+	const char *p;
 	int id;
 	
 	if (!g_ascii_strcasecmp (folder_name, "INBOX")) {
@@ -864,6 +856,9 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
+	if (selected && !strcmp (folder_name, selected->full_name))
+		ic0 = camel_imap4_engine_queue (engine, NULL, "CLOSE\r\n");
+	
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_imap4_engine_queue (engine, NULL, "DELETE %S\r\n", utf7_name);
 	g_free (utf7_name);
@@ -872,18 +867,42 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 		;
 	
 	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
-		camel_exception_xfer (ex, &ic->ex);
+		if (ic0 && ic0->status != CAMEL_IMAP4_COMMAND_COMPLETE)
+			camel_exception_xfer (ex, &ic0->ex);
+		else
+			camel_exception_xfer (ex, &ic->ex);
+		
+		if (ic0 != NULL)
+			camel_imap4_command_unref (ic0);
+		
 		camel_imap4_command_unref (ic);
 		CAMEL_SERVICE_UNLOCK (store, connect_lock);
 		return;
 	}
 	
+	if (ic0 != NULL)
+		camel_imap4_command_unref (ic0);
+	
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
 		/* deleted */
-		/*fi = imap4_build_folder_info (store, folder_name);
+		url = camel_url_copy (engine->url);
+		camel_url_set_fragment (url, folder_name);
+		
+		p = strrchr (folder_name, '/');
+		
+		fi = g_malloc0 (sizeof (CamelFolderInfo));
+		fi->full_name = g_strdup (folder_name);
+		fi->name = g_strdup (p ? p + 1: folder_name);
+		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+		camel_url_free (url);
+		fi->flags = 0;
+		fi->unread = -1;
+		fi->total = -1;
+		
 		camel_object_trigger_event (store, "folder_deleted", fi);
-		camel_folder_info_free (fi);*/
+		
+		camel_folder_info_free (fi);
 		break;
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
@@ -906,7 +925,58 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 static void
 imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_name, CamelException *ex)
 {
-
+	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	char *old_uname, *new_uname;
+	CamelIMAP4Command *ic;
+	int id;
+	
+	if (!g_ascii_strcasecmp (old_name, "INBOX")) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot rename folder `%s' to `%s': Special folder"),
+				      old_name, new_name);
+		
+		return;
+	}
+	
+	CAMEL_SERVICE_LOCK (store, connect_lock);
+	
+	old_uname = imap4_folder_utf7_name (store, old_name, '\0');
+	new_uname = imap4_folder_utf7_name (store, new_name, '\0');
+	
+	ic = camel_imap4_engine_queue (engine, NULL, "RENAME %S %S\r\n", old_uname, new_uname);
+	g_free (old_uname);
+	g_free (new_uname);
+	
+	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
+		;
+	
+	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
+		camel_exception_xfer (ex, &ic->ex);
+		camel_imap4_command_unref (ic);
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+		return;
+	}
+	
+	switch (ic->result) {
+	case CAMEL_IMAP4_RESULT_OK:
+		/* FIXME: need to update state on the renamed folder object */
+		break;
+	case CAMEL_IMAP4_RESULT_NO:
+		/* FIXME: would be good to save the NO reason into the err message */
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot rename folder `%s' to `%s': Invalid mailbox name"),
+				      old_name, new_name);
+		break;
+	case CAMEL_IMAP4_RESULT_BAD:
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot rename folder `%s' to `%s': Bad command"),
+				      old_name, new_name);
+		break;
+	}
+	
+	camel_imap4_command_unref (ic);
+	
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 }
 
 static int
@@ -933,15 +1003,70 @@ list_remove_duplicates (GPtrArray *array)
 	}
 }
 
-static CamelFolderInfo *
-imap4_build_folder_info (CamelIMAP4Engine *engine, guint32 flags, GPtrArray *array, const char *top)
+static void
+imap4_status (CamelStore *store, CamelFolderInfo *fi)
 {
+	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	camel_imap4_status_attr_t *attr, *next;
+	camel_imap4_status_t *status;
+	CamelIMAP4Command *ic;
+	GPtrArray *array;
+	char *mailbox;
+	int id, i;
+	
+	mailbox = imap4_folder_utf7_name (store, fi->full_name, '\0');
+	ic = camel_imap4_engine_queue (engine, NULL, "STATUS %S (MESSAGES UNSEEN)\r\n", mailbox);
+	g_free (mailbox);
+	
+	camel_imap4_command_register_untagged (ic, "STATUS", camel_imap4_untagged_status);
+	ic->user_data = array = g_ptr_array_new ();
+	
+	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
+		;
+	
+	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
+		camel_imap4_command_unref (ic);
+		g_ptr_array_free (array, TRUE);
+		return;
+	}
+	
+	for (i = 0; i < array->len; i++) {
+		status = array->pdata[i];
+		attr = status->attr_list;
+		while (attr != NULL) {
+			next = attr->next;
+			if (attr->type == CAMEL_IMAP4_STATUS_MESSAGES)
+				fi->total = attr->value;
+			else if (attr->type == CAMEL_IMAP4_STATUS_UNSEEN)
+				fi->unread = attr->value;
+			g_free (attr);
+			attr = next;
+		}
+		
+		g_free (status->mailbox);
+		g_free (status);
+	}
+	
+	camel_imap4_command_unref (ic);
+	g_ptr_array_free (array, TRUE);
+}
+
+static CamelFolderInfo *
+imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtrArray *array)
+{
+	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelFolder *folder = (CamelFolder *) engine->folder;
 	camel_imap4_list_t *list;
 	CamelFolderInfo *fi;
 	char *name, *p;
 	CamelURL *url;
 	int i;
 	
+	if (array->len == 0) {
+		g_ptr_array_free (array, TRUE);
+		return NULL;
+	}
+	
 	g_ptr_array_sort (array, (GCompareFunc) list_sort);
 	
 	list_remove_duplicates (array);
@@ -959,19 +1084,39 @@ imap4_build_folder_info (CamelIMAP4Engine *engine, guint32 flags, GPtrArray *arr
 			p++;
 		}
 		
+		p = strrchr (name, '/');
 		camel_url_set_fragment (url, name);
 		
 		fi->full_name = name;
-		p = strrchr (name, '/');
 		fi->name = g_strdup (p ? p + 1: name);
-		fi->path = g_strdup_printf ("/%s", name);
-		fi->uri = camel_url_to_string (url, 0);
+		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
 		fi->flags = list->flags;
-		
-		/* FIXME: use STATUS to get these values if requested */
 		fi->unread = -1;
 		fi->total = -1;
 		
+		if (!(flags & CAMEL_STORE_FOLDER_INFO_FAST)) {
+			if (folder && !strcmp (folder->full_name, fi->full_name)) {
+				/* can't STATUS this folder since it is SELECTED, besides - it would be wasteful */
+				CamelMessageInfo *info;
+				int index;
+				
+				fi->total = camel_folder_summary_count (folder->summary);
+				
+				fi->unread = 0;
+				for (index = 0; index < fi->total; index++) {
+					if (!(info = camel_folder_summary_index (folder->summary, index)))
+						continue;
+					
+					if ((info->flags & CAMEL_MESSAGE_SEEN) == 0)
+						fi->unread++;
+					
+					camel_folder_summary_info_free (folder->summary, info);
+				}
+			} else {
+				imap4_status (store, fi);
+			}
+		}
+		
 		g_free (list->name);
 		g_free (list);
 		
@@ -991,17 +1136,22 @@ static CamelFolderInfo *
 imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelIMAP4Command *ic, *ic0 = NULL;
 	CamelFolderInfo *fi = NULL;
 	camel_imap4_list_t *list;
-	CamelIMAP4Command *ic;
 	GPtrArray *array;
 	const char *cmd;
 	char *pattern;
+	char wildcard;
 	int id, i;
 	
-	if (engine == NULL) {
+	CAMEL_SERVICE_LOCK (store, connect_lock);
+	
+	if (engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED) {
 		if (!camel_service_connect ((CamelService *) store, ex))
 			return NULL;
+		
+		engine = ((CamelIMAP4Store *) store)->engine;
 	}
 	
 	if (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED)
@@ -1012,24 +1162,57 @@ imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelE
 	if (top == NULL)
 		top = "";
 	
-	CAMEL_SERVICE_LOCK (store, connect_lock);
+	wildcard = (flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE) ? '*' : '%';
+	pattern = imap4_folder_utf7_name (store, top, wildcard);
+	array = g_ptr_array_new ();
 	
-	pattern = imap4_folder_utf7_name (store, top, (flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE) ? '*' : '%');
-	ic = camel_imap4_engine_queue (engine, NULL, "%s "" %S\r\n", cmd, pattern);
+	if (*top != '\0') {
+		size_t len;
+		char sep;
+		
+		len = strlen (pattern);
+		sep = pattern[len - 2];
+		pattern[len - 2] = '\0';
+		
+		ic0 = camel_imap4_engine_queue (engine, NULL, "%s \"\" %S\r\n", cmd, pattern);
+		camel_imap4_command_register_untagged (ic0, cmd, camel_imap4_untagged_list);
+		ic0->user_data = array;
+		
+		pattern[len - 2] = sep;
+	}
+	
+	ic = camel_imap4_engine_queue (engine, NULL, "%s \"\" %S\r\n", cmd, pattern);
 	camel_imap4_command_register_untagged (ic, cmd, camel_imap4_untagged_list);
-	ic->user_data = array = g_ptr_array_new ();
+	ic->user_data = array;
 	
 	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
 	
 	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
-		camel_exception_xfer (ex, &ic->ex);
+		if (ic0 && ic0->status != CAMEL_IMAP4_COMMAND_COMPLETE)
+			camel_exception_xfer (ex, &ic0->ex);
+		else
+			camel_exception_xfer (ex, &ic->ex);
+		
+		if (ic0 != NULL)
+			camel_imap4_command_unref (ic0);
 		camel_imap4_command_unref (ic);
+		
+		for (i = 0; i < array->len; i++) {
+			list = array->pdata[i];
+			g_free (list->name);
+			g_free (list);
+		}
+		
 		g_ptr_array_free (array, TRUE);
 		g_free (pattern);
+		
 		goto done;
 	}
 	
+	if (ic0 != NULL)
+		camel_imap4_command_unref (ic0);
+	
 	if (ic->result != CAMEL_IMAP4_RESULT_OK) {
 		camel_imap4_command_unref (ic);
 		
@@ -1053,7 +1236,7 @@ imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelE
 	
 	g_free (pattern);
 	
-	fi = imap4_build_folder_info (engine, flags, array, top);
+	fi = imap4_build_folder_info (store, top, flags, array);
 	
  done:
 	
@@ -1067,7 +1250,10 @@ imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcepti
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
 	CamelIMAP4Command *ic;
+	CamelFolderInfo *fi;
 	char *utf7_name;
+	CamelURL *url;
+	const char *p;
 	int id;
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
@@ -1089,10 +1275,22 @@ imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcepti
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
 		/* subscribed */
-		/*fi = imap4_build_folder_info (store, folder_name);
-		  fi->flags |= CAMEL_FOLDER_NOCHILDREN;
-		  camel_object_trigger_event (store, "folder_subscribed", fi);
-		  camel_folder_info_free (fi);*/
+		url = camel_url_copy (engine->url);
+		camel_url_set_fragment (url, folder_name);
+		
+		p = strrchr (folder_name, '/');
+		
+		fi = g_malloc0 (sizeof (CamelFolderInfo));
+		fi->full_name = g_strdup (folder_name);
+		fi->name = g_strdup (p ? p + 1: folder_name);
+		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+		camel_url_free (url);
+		fi->flags = CAMEL_FOLDER_NOCHILDREN;
+		fi->unread = -1;
+		fi->total = -1;
+		
+		camel_object_trigger_event (store, "folder_subscribed", fi);
+		camel_folder_info_free (fi);
 		break;
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
@@ -1117,7 +1315,10 @@ imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExcep
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
 	CamelIMAP4Command *ic;
+	CamelFolderInfo *fi;
 	char *utf7_name;
+	CamelURL *url;
+	const char *p;
 	int id;
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
@@ -1139,9 +1340,22 @@ imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExcep
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
 		/* unsubscribed */
-		/*fi = imap4_build_folder_info (store, folder_name);
-		  camel_object_trigger_event (store, "folder_unsubscribed", fi);
-		  camel_folder_info_free (fi);*/
+		url = camel_url_copy (engine->url);
+		camel_url_set_fragment (url, folder_name);
+		
+		p = strrchr (folder_name, '/');
+		
+		fi = g_malloc0 (sizeof (CamelFolderInfo));
+		fi->full_name = g_strdup (folder_name);
+		fi->name = g_strdup (p ? p + 1: folder_name);
+		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+		camel_url_free (url);
+		fi->flags = 0;
+		fi->unread = -1;
+		fi->total = -1;
+		
+		camel_object_trigger_event (store, "folder_unsubscribed", fi);
+		camel_folder_info_free (fi);
 		break;
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
@@ -1165,11 +1379,20 @@ static void
 imap4_noop (CamelStore *store, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelFolder *folder = (CamelFolder *) engine->folder;
 	CamelIMAP4Command *ic;
 	int id;
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
+	if (folder) {
+		camel_folder_sync (folder, FALSE, ex);
+		if (camel_exception_is_set (ex)) {
+			CAMEL_SERVICE_UNLOCK (store, connect_lock);
+			return;
+		}
+	}
+	
 	ic = camel_imap4_engine_queue (engine, NULL, "NOOP\r\n");
 	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
@@ -1179,5 +1402,8 @@ imap4_noop (CamelStore *store, CamelException *ex)
 	
 	camel_imap4_command_unref (ic);
 	
+	if (folder && !camel_exception_is_set (ex))
+		camel_imap4_summary_flush_updates (folder->summary, ex);
+	
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 }
